\ blanket stitch engine
\ public domain

include engine_common.fs

( direct page )

$f0 constant voice0_count
$f1 constant voice1_count
$f2 constant voice2_count
$f3 constant voice3_count
$f4 constant voice0_freq
$f5 constant voice1_freq
$f6 constant voice2_freq
$f7 constant voice3_freq
$f8 constant step_count
$f9 constant pulse_flag
$fa constant s_save
\          x speed_count
\          s speed_freq

( header row )
$0 constant header.speed
$1 constant header%

( pattern row )
$0 constant pattern.length
$1 constant pattern.note0
$2 constant pattern.note1
$3 constant pattern.note2
$4 constant pattern.note3
$5 constant pattern%

\ delay 2+ cycles
: delay-cycles ( cycles -- )
  dup $2 mod if
    $00 @d lda,
    $3 -
  then
  0 ?do
    nop,
  $2 +loop ;

mainseg switch-segment

player_start torg

label start
  player_entry_table% tallot

label playloop
  $00 # lda, \ 2

  \ voice 0
  voice0_count @d dec, \ 5
  zero if,
                         \ 2
    voice0_freq  @d lda, \ 3
    voice0_count @d sta, \ 3

    pulse_flag   @d sta, \ 3
    nop,                 \ 2
                         \ 3
  else,
                               \ 3
    pulse_flag @d sta,         \ 3

    pattern.note0 # ldy,       \ 2
    pattern_pointer (d),y lda, \ 5 + 1
    voice0_freq @d sta,        \ 3
  then,

  \ voice 1
  voice1_count @d dec, \ 5
  zero ~if,
                               \ 2
    pattern.note1 # ldy,       \ 2
    pattern_pointer (d),y lda, \ 5 + 1
    voice1_freq @d sta,        \ 3
                               \ 3
  else,
                         \ 3
    voice1_freq  @d lda, \ 3
    voice1_count @d sta, \ 3

    pulse_flag   @d ora, \ 3
    pulse_flag   @d sta, \ 3
  then,

  \ voice 2
  voice2_count @d dec, \ 5
  zero ~if,
                               \ 2
    pattern.note2 # ldy,       \ 2
    pattern_pointer (d),y lda, \ 5 + 1
    voice2_freq @d sta,        \ 3
                               \ 3
  else,
                         \ 3
    voice2_freq  @d lda, \ 3
    voice2_count @d sta, \ 3

    pulse_flag   @d ora, \ 3
    pulse_flag   @d sta, \ 3
  then,

  \ voice 3
  voice3_count @d dec, \ 5
  zero ~if,
                               \ 2
    pattern.note3 # ldy,       \ 2
    pattern_pointer (d),y lda, \ 5 + 1
    voice3_freq @d sta,        \ 3
                               \ 3
  else,
                         \ 3
    voice3_freq  @d lda, \ 3
    voice3_count @d sta, \ 3

    pulse_flag   @d ora, \ 3
    pulse_flag   @d sta, \ 3
  then,

: control, ( -- ) \ 31
  dex, \ 2
  zero ~if,
    \ 2
    #5 delay-cycles

    step_count @d lda, \ 3
    zero ~if,
      \ 2
      #11 delay-cycles
      \ 3
    else,
                                 \ 3
      $00 # ldy,                 \ 2
      pattern_pointer (d),y lda, \ 5 + 1
      step_count @d sta,         \ 3

      zero if,                   \ 3
        s_save @d ldx,
        txs,
        rts,
      then,
    then,

    \ 3
  else,
    \ 3
    tsx, \ 2

    step_count @d dec, \ 5
    zero ~if,
      \ 2
      #14 delay-cycles
      \ 3
    else,
      \ 3

      \ this is the only section of code which modifies the carry bit.
      \ the addition never produces a carry, since that would mean the
      \ song data wraps around the end of memory. there is no need to
      \ clc before the addition since the previous addition cleared
      \ carry already.

      pattern_pointer0 @d lda, \ 3
      pattern% # adc,          \ 2
      pattern_pointer0 @d sta, \ 3

      pattern_pointer1 @d lda, \ 3
      $00 # adc,               \ 2
      pattern_pointer1 @d sta, \ 3

    then,
  then, ;

  pulse_flag @d lda, \ 3
  zero ~if,
                               \ 2
    speaker_out @a sta, fixup, \ 4
    control,                   \ 31
    speaker_out @a sta, fixup, \ 4

    playloop @a jmp,           \ 3
  then,
                   \ 3

  keyboard_data_strobe @a lda, \ 4
  sign if,                     \ 3
    keyboard_clear_strobe @a sta,

    s_save @d ldx,
    txs,
    rts,
  then,

  control,         \ 31

  playloop @a jmp, \ 3

player_start @a jmphere
  \ move to first row
  \ save header location
  clc,
  pattern_pointer0 @d lda, header_pointer0 @d sta,
  header% # adc,
  pattern_pointer0 @d sta,

  pattern_pointer1 @d lda, header_pointer1 @d sta,
  $00 # adc,
  pattern_pointer1 @d sta,

player_start_header @a jmphere
  \ the player loop expects carry to be clear
  clc,

  \ save stack pointer
  tsx,
  s_save @d stx,

  \ load header values
  $00 # ldy,
  header_pointer (d),y lda,
  tax, \ speed_count
  txs, \ speed_freq

  \ load first row values
  $00 # ldy,
  pattern_pointer (d),y lda,
  step_count @d sta,

  iny,
  pattern_pointer (d),y lda,
  voice0_count @d sta,
  voice0_freq @d sta,

  iny,
  pattern_pointer (d),y lda,
  voice1_count @d sta,
  voice1_freq @d sta,

  iny,
  pattern_pointer (d),y lda,
  voice2_count @d sta,
  voice2_freq @d sta,

  iny,
  pattern_pointer (d),y lda,
  voice3_count @d sta,
  voice3_freq @d sta,

  playloop @a jmp,

there start - constant length

code-segment bank new-segment constant mainseg.out
mainseg.out switch-segment
start length mainseg copy-range

s" BLANKET"       mainseg.out write-segment-by-filename
s" BLANKET.FIXUP" fixupseg    write-segment-by-filename

length cr hex.

cr .s
cr bye

