\ running stitch engine
\ public domain

include engine_common.fs

( direct page )

$f0 constant voice0_count
$f1 constant voice1_count
$f2 constant voice2_count
$f3 constant voice3_count
$f4 constant voice0_freq
$f5 constant voice1_freq
$f6 constant voice2_freq
$f7 constant voice3_freq
$f8 constant step_count
$f9 constant pulse_flag
$fa constant speed_count
$fb constant speed_freq

( header row )
$0 constant header.speed
$1 constant header%

( pattern row )
$0 constant pattern.length
$1 constant pattern.note0
$2 constant pattern.note1
$3 constant pattern.note2
$4 constant pattern.note3
$5 constant pattern%

\ delay 2+ cycles
: delay-cycles ( cycles -- )
  dup $2 mod if
    $00 @d lda,
    $3 -
  then
  0 ?do
    nop,
  $2 +loop ;

mainseg switch-segment

player_start torg

label start
  player_entry_table% tallot

: voice, { count freq -- }
  count @d dec,        \ 5
  zero if,
                       \ 2
    freq  @d lda,      \ 3
    count @d sta,      \ 3

    pulse_flag @d ora, \ 3
    pulse_flag @d sta, \ 3
  then,                \ 3
;

label delay16
  nop,
  nop,
  rts,

label delay64
  delay16 @a jsr,

label delay48
  delay16 @a jsr,

label delay32
  delay16 @a jsr,
  rts,

label playloop
  $00 # lda, \ 2
  pulse_flag @d sta, \ 3

  \ voices
  voice0_count voice0_freq voice,
  voice1_count voice1_freq voice,
  voice2_count voice2_freq voice,
  voice3_count voice3_freq voice,

  \ control
  speed_count @d dec, \ 5
  zero if,
    \ 2
    speed_freq  @d lda, \ 3
    speed_count @d sta, \ 3

    step_count @d dec, \ 5
    zero if,
      \ 2

      clc,                       \ 2
      pattern_pointer0 @d lda,   \ 3
      pattern% # adc,            \ 2
      pattern_pointer0 @d sta,   \ 3

      pattern_pointer1 @d lda,   \ 3
      $00 # adc,                 \ 2
      pattern_pointer1 @d sta,   \ 3

      pattern.length # ldy,      \ 2
      pattern_pointer (d),y lda, \ 5 + 1
      step_count @d sta,         \ 3

      zero if,                   \ 3
        rts,
      then,

      pattern.note0 # ldy,       \ 2
      pattern_pointer (d),y lda, \ 5 + 1
      voice0_freq @d sta,        \ 3

      pattern.note1 # ldy,       \ 2
      pattern_pointer (d),y lda, \ 5 + 1
      voice1_freq @d sta,        \ 3

      pattern.note2 # ldy,       \ 2
      pattern_pointer (d),y lda, \ 5 + 1
      voice2_freq @d sta,        \ 3

      pattern.note3 # ldy,       \ 2
      pattern_pointer (d),y lda, \ 5 + 1
      voice3_freq @d sta,        \ 3
    then, \ 3
  then, \ 3

  pulse_flag @d lda, \ 3
  zero ~if,
                               \ 2
    speaker_out @a sta, fixup, \ 4

    \ longer pulses for lower frequencies
    delay16 @a jsr,
    $20 # cmp,
    carry if,
      delay16 @a jsr,
      $40 # cmp,
      carry if,
        delay32 @a jsr,
        $80 # cmp,
        carry if,
          delay64 @a jsr,
        then,
      then,
    then,

    speaker_out @a sta, fixup, \ 4
  then, \ 3

  keyboard_data_strobe @a lda, \ 4
  sign if,                     \ 3
    keyboard_clear_strobe @a sta,
    rts,
  then,

  playloop @a jmp, \ 3

player_start @a jmphere
  \ move to first row
  \ save header location
  clc,
  pattern_pointer0 @d lda, header_pointer0 @d sta,
  header% # adc,
  pattern_pointer0 @d sta,

  pattern_pointer1 @d lda, header_pointer1 @d sta,
  $00 # adc,
  pattern_pointer1 @d sta,

player_start_header @a jmphere
  \ the player loop expects carry to be clear
  clc,

  \ load header values
  $00 # ldy,
  header_pointer (d),y lda,
  speed_count @d sta,
  speed_freq @d sta,

  \ load first row values
  $00 # ldy,
  pattern_pointer (d),y lda,
  step_count @d sta,

  iny,
  pattern_pointer (d),y lda,
  voice0_count @d sta,
  voice0_freq @d sta,

  iny,
  pattern_pointer (d),y lda,
  voice1_count @d sta,
  voice1_freq @d sta,

  iny,
  pattern_pointer (d),y lda,
  voice2_count @d sta,
  voice2_freq @d sta,

  iny,
  pattern_pointer (d),y lda,
  voice3_count @d sta,
  voice3_freq @d sta,

  playloop @a jmp,

there start - constant length

code-segment bank new-segment constant mainseg.out
mainseg.out switch-segment
start length mainseg copy-range

s" RUNNING"       mainseg.out write-segment-by-filename
s" RUNNING.FIXUP" fixupseg    write-segment-by-filename

length cr hex.

cr .s
cr bye

